using UnityEngine;
using UnityEngine.Animations;

public class InvisiblePinBehaviour : PinBehaviour
{
  public float BreakingThreshold = float.PositiveInfinity;
  [SkipSerialisation]
  public PhysicalBehaviour PhysicalBehaviour;
  public PhysicalBehaviour Other;
  public Vector2 LocalAnchor;
  [SkipSerialisation]
  public Transform SpriteChild;
  [SkipSerialisation]
  public SpriteRenderer SpriteRenderer;
  [SkipSerialisation]
  public HingeJoint2D Joint;
  public bool UsedToHaveConnectedBody;
  [HideInInspector]
  public string PinSpritePath = "Sprites/Pin";
  private float initialBreakForce = float.PositiveInfinity;

  public bool AttachedToWall => (bool) (Object) this.Other;

  private void Awake() => this.PhysicalBehaviour = this.GetComponent<PhysicalBehaviour>();

  private void Start() => this.CreateJoint();

  protected void CreateJoint()
  {
    this.Joint = this.gameObject.AddComponent<HingeJoint2D>();
    this.Joint.breakForce = Utils.CalculateBreakForceForCable((AnchoredJoint2D) this.Joint, this.BreakingThreshold);
    this.initialBreakForce = this.Joint.breakForce;
    this.Joint.anchor = this.LocalAnchor;
    if (this.AttachedToWall)
      this.Joint.connectedBody = this.Other.rigidbody;
    GameObject gameObject = new GameObject("Pin");
    this.SpriteChild = gameObject.transform;
    this.SpriteChild.position = this.transform.TransformPoint((Vector3) this.LocalAnchor);
    this.SpriteChild.localScale = new Vector3(1f, 1f, 1f);
    ParentConstraint parentConstraint = gameObject.AddComponent<ParentConstraint>();
    parentConstraint.AddSource(new ConstraintSource()
    {
      sourceTransform = this.transform,
      weight = 1f
    });
    parentConstraint.SetTranslationOffset(0, (Vector3) this.LocalAnchor);
    parentConstraint.constraintActive = true;
    SpriteRenderer component1;
    bool component2 = this.TryGetComponent<SpriteRenderer>(out component1);
    this.SpriteRenderer = gameObject.AddComponent<SpriteRenderer>();
    this.SpriteRenderer.sortingLayerName = component2 ? component1.sortingLayerName : "Top";
    this.SpriteRenderer.sortingOrder = component2 ? component1.sortingOrder + 1 : 100;
    this.SpriteRenderer.sprite = Resources.Load<Sprite>(this.PinSpritePath);
    this.SpriteRenderer.color = new Color(0f, 0f, 0f, 0f);
    this.SpriteRenderer.gameObject.layer = LayerMask.NameToLayer("Default");
    CircleCollider2D circleCollider2D = gameObject.AddComponent<CircleCollider2D>();
    circleCollider2D.isTrigger = true;
    circleCollider2D.radius = 0.07142857f;
    this.collider = (Collider2D) circleCollider2D;
    gameObject.AddComponent<Optout>();
  }

  private void Update()
  {
    if (Global.main.GetPausedMenu())
      return;
    bool flag = (bool) (Object) this.Other && this.Other.isDisintegrated;
    if (((!((Object) this.Joint.connectedBody == (Object) null) ? 0 : (this.UsedToHaveConnectedBody ? 1 : 0)) | (flag ? 1 : 0)) != 0)
    {
      Object.Destroy((Object) this);
    }
    else
    {
      if (!(bool) (Object) this.SpriteChild)
        return;
      this.CheckMouseInput();
    }
  }

  private void FixedUpdate()
  {
    if ((double) this.initialBreakForce == double.PositiveInfinity)
      return;
    float a = 0.0f;
    if ((bool) (Object) this.PhysicalBehaviour)
      a = this.PhysicalBehaviour.BurnProgress;
    if ((bool) (Object) this.Other)
      a = Mathf.Max(a, this.Other.BurnProgress);
    this.Joint.breakForce = this.initialBreakForce * (1f - a);
  }

  public override void OnMouseOverlapEvent(bool overlap)
  {
    base.OnMouseOverlapEvent(overlap);
    if (overlap)
    {
      this.SpriteRenderer.color = Color.red;
      this.SpriteRenderer.gameObject.layer = LayerMask.NameToLayer("ScreenUI");
    }
    else
    {
      this.SpriteRenderer.color = new Color(0f, 0f, 0f, 0f);
      this.SpriteRenderer.gameObject.layer = LayerMask.NameToLayer("Default");
    }
  }

  private void OnJointBreak2D(Joint2D broken)
  {
    if (!((Object) broken == (Object) this.Joint))
      return;
    Object.Destroy((Object) this);
  }

  protected override void OnDestroy()
  {
    base.OnDestroy();
    if ((bool) (Object) this.SpriteChild)
    {
      Object.Destroy((Object) this.SpriteChild.gameObject);
      Debug.Log((object) "wedawd");
    }
    Object.Destroy((Object) this.Joint);
    Object.Destroy((Object) this);
  }

  private void OnDisable()
  {
    if (!(bool) (Object) this.SpriteChild)
      return;
    this.SpriteChild.gameObject.SetActive(false);
  }

  private void OnEnable()
  {
    if (!(bool) (Object) this.SpriteChild)
      return;
    this.SpriteChild.gameObject.SetActive(true);
  }

  public virtual void Slice()
  {
    this.Joint.breakForce = 0.0f;
    this.Joint.breakTorque = 0.0f;
  }

  protected override Bounds GetVisualBounds() => this.SpriteRenderer.bounds;
}