using UnityEngine;

public class InvisiblePinTool : ToolBehaviour
{
  private readonly Collider2D[] buffer = new Collider2D[8];
  private int bufferLength;
  private PhysicalBehaviour firstFound;
  private PhysicalBehaviour secondFound;

  public override void OnSelect()
  {
    if (DialogBox.IsAnyDialogboxOpen || Global.ActiveUiBlock)
      return;
    this.GetFoundBodies();
    if (!(bool) (Object) this.firstFound && !(bool) (Object) this.secondFound)
      return;
    this.CreatePin((Vector2) this.firstFound.transform.InverseTransformPoint(Global.main.MousePosition), (bool) (Object) this.firstFound ? this.firstFound : this.secondFound, (bool) (Object) this.firstFound ? this.secondFound : (PhysicalBehaviour) null, this.GetBreakingThreshold());
  }

  private void GetFoundBodies()
  {
    this.firstFound = (PhysicalBehaviour) null;
    this.secondFound = (PhysicalBehaviour) null;
    this.bufferLength = Physics2D.OverlapPointNonAlloc((Vector2) Global.main.MousePosition, this.buffer);
    for (int index = 0; index < this.bufferLength; ++index)
    {
      PhysicalBehaviour component;
      if (this.buffer[index].TryGetComponent<PhysicalBehaviour>(out component))
      {
        if (!(bool) (Object) this.firstFound || (Object) this.firstFound == (Object) component)
        {
          this.firstFound = component;
        }
        else
        {
          if ((bool) (Object) this.secondFound && !((Object) this.secondFound == (Object) component))
            break;
          this.secondFound = component;
        }
      }
    }
  }

  public override void OnFixedHold()
  {
  }

  public override void OnHold()
  {
  }

  public override void OnDeselect()
  {
  }

  public override void OnToolChosen()
  {
  }

  public override void OnToolUnchosen()
  {
  }

  protected virtual float GetBreakingThreshold() => float.PositiveInfinity;

  public virtual void CreatePin(
    Vector2 anchor,
    PhysicalBehaviour a,
    PhysicalBehaviour b = null,
    float breakThreshold = float.PositiveInfinity)
  {
    InvisiblePinBehaviour pin = a.gameObject.AddComponent<InvisiblePinBehaviour>();
    UndoControllerBehaviour.RegisterAction((IUndoableAction) new ObjectCreationAction((Object) pin, "pin"));
    pin.LocalAnchor = anchor;
    pin.Other = b;
    pin.BreakingThreshold = breakThreshold;
    pin.UsedToHaveConnectedBody = (bool) (Object) b;
    this.OnPinCreation(pin);
  }

  protected virtual void OnPinCreation(InvisiblePinBehaviour pin)
  {
  }
}